import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JCheckBox;
import javax.swing.JScrollBar;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.geom.AffineTransform;
import java.awt.image.*;
import java.io.*;

import javax.imageio.*;
import javax.swing.JPanel;

/**
 * @author Markus Bttcher Created on 16.12.2004 functions to filter images are
 *         implemented in MyPanel
 */
public class MyFrame extends JFrame {
	private static final long serialVersionUID = 1L;
	private MyFrame linkedWindow = null;
	private MyPanel mypanel = null;
	private javax.swing.JPanel jContentPane = null;
	private JScrollPane jScrollPane = null;
	private JScrollBar jScrollBar_h = null;
	private JScrollBar jScrollBar_v = null; // @jve:decl-index=0:visual-constraint="716,179"

	/**
	 * This is the default constructor
	 */
	public MyFrame() {
		super();
		initialize();
	}

	/**
	 * set the title of the frame
	 * 
	 * @param s
	 *            title of the frame
	 */
	public MyFrame(String s) {
		super();
		initialize();
		this.setTitle(s);
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setLocation(400, 400);
		this.setSize(600, 600);
		this.setContentPane(getJContentPane());
		this.setTitle("JFrame");
		this.setVisible(true);
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new javax.swing.JPanel();
			jContentPane.setLayout(new java.awt.BorderLayout());
			jContentPane.add(getJScrollPane(), java.awt.BorderLayout.CENTER);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setViewportView(getmyPanel());
			jScrollBar_h = jScrollPane.getHorizontalScrollBar();
			jScrollBar_h
					.addAdjustmentListener(new java.awt.event.AdjustmentListener() {
						public void adjustmentValueChanged(
								java.awt.event.AdjustmentEvent e) {
							if (linkedWindow != null) {
								linkedWindow.setHorizontalScroll(jScrollBar_h
										.getValue());
							}
						}
					});
			jScrollBar_v = jScrollPane.getVerticalScrollBar();
			jScrollBar_v
					.addAdjustmentListener(new java.awt.event.AdjustmentListener() {
						public void adjustmentValueChanged(
								java.awt.event.AdjustmentEvent e) {
							if (linkedWindow != null) {
								linkedWindow.setVerticalScroll(jScrollBar_v
										.getValue());
							}
						}
					});
		}
		return jScrollPane;
	}

	private MyPanel getmyPanel() {
		if (mypanel == null) {
			mypanel = new MyPanel();
		}
		return mypanel;
	}

	public void setImage(String image, int filter) {
		mypanel.setImage(image, filter);
	}

	public void setImage(String image) {
		mypanel.setImage(image);
	}

	public void setJCheckBox(JCheckBox jcb) {
		mypanel.setJCheckBox(jcb);
	}

	public int getSelectedX() {
		return mypanel.selectedX;
	}

	public int getSelectedY() {
		return mypanel.selectedY;
	}

	public void runFilter(double radius, double sigma) {
		mypanel.runFilter_seperate(radius, sigma);
	}

	public void saveImage(String s) {
		mypanel.saveImage(s);
	}

	public void check(int value1, int value2) {
		mypanel.check(value1, value2);
	}

	public void erode() {
		mypanel.erode();
	}

	public void dilate() {
		mypanel.dilate();
	}

	public void skeletonize() {
		mypanel.skeletonizeZHANG2();
	}

	public void setLinkedWindow(MyFrame lWindow) {
		this.linkedWindow = lWindow;
	}

	public void setHorizontalScroll(int value) {
		jScrollBar_h.setValue(value);
	}

	public void setVerticalScroll(int value) {
		jScrollBar_v.setValue(value);
	}

	public void undo() {
		mypanel.undo();
	}

	public void setRealSize(boolean realSize) {
		mypanel.setRealSize(realSize);
	}

	public void disposeMyHistogram() {
		mypanel.disposeMyHistogram();
	}

	public void countPixels() {
		mypanel.countPixels();
	}

	public void setCalibration(double x_pixel, double x_length, double y_pixel,
			double y_length) {
		mypanel.setCalibration(x_pixel, x_length, y_pixel, y_length);
	}

	public void setHistoLocation(int x, int y) {
		mypanel.setHistoLocation(x, y);
	}

	public void setHistoSize(int width, int height) {
		mypanel.setHistoSize(width, height);
	}
	public void setVisibleHistogram(boolean showhisto){
			mypanel.setVisibleHistogram(showhisto);
	}
} // @jve:decl-index=0:visual-constraint="10,10"

/*
 * Created on 16.12.2004 @author Markus Bttcher
 */
class MyPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	private double n_Pixel_X = 0;

	private double n_Laenge_X = 0;

	private double n_Pixel_Y = 0;

	private double n_Laenge_Y = 0;

	private String imageName = ""; // @jve:decl-index=0:

	private Graphics2D scalable = null; // @jve:decl-index=0:

	private AffineTransform scaling = null; // @jve:decl-index=0:

	private boolean realSize = true;

	private double scale = 0;

	private double aspectW = 0;

	private double aspectI = 0;

	private int scaledWidth = 0;

	private int scaledHeight = 0;

	private MyHistogram myHistogram = null;

	public int threshold = 0;

	public int selectedX = 0;

	public int selectedY = 0;

	private JCheckBox jcb = null;

	// original image
	private Image oImage = null; // @jve:decl-index=0:

	// Dimensions of the original image
	private int iWidth = 0;

	private int iHeight = 0;

	// contains the original pixel values after grayscale conversion
	private int[] pixels_original = null;

	// the arrays that are used to store the pixel values before and after
	// processing
	private int[] pixels_before_function = null;

	private int[] pixels_after_function = null;

	// result image that is displayed
	private Image rImage = null; // @jve:decl-index=0:

	// pixel values of the result image
	private int[] pixels_result = null;

	/**
	 * @param realSize
	 *            the realSize to set
	 */
	public void setRealSize(boolean realSize) {
		this.realSize = realSize;
		calculateDimension();
	}

	private void calculateDimension() {
		if (realSize) {
			setSize(iWidth, iHeight);
			setPreferredSize(new Dimension(iWidth, iHeight));
		} else {
			aspectI = (double) iWidth / (double) iHeight;
			aspectW = getVisibleRect().getWidth()
					/ getVisibleRect().getHeight();
			if (aspectI >= aspectW) {
				scale = 1.0 / iWidth * getVisibleRect().getWidth();
			} else {
				scale = 1.0 / iHeight * getVisibleRect().getHeight();
			}
			scaledWidth = Math.round((float) scale * getWidth());
			scaledHeight = Math.round((float) scale * getHeight());
			setSize(scaledWidth, scaledHeight);
			setPreferredSize(new Dimension(scaledWidth, scaledHeight));
		}
		repaint();
	}

	/**
	 * This is the default constructor
	 */
	public MyPanel() {
		super();
		initialize();
	}

	private void initialize() {
		this.addMouseListener(new java.awt.event.MouseAdapter() {
			@Override
			public void mouseClicked(java.awt.event.MouseEvent e) {
				selectedX = e.getX();
				selectedY = e.getY();
				jcb.setSelected(true);
			}
		});
	}

	@Override
	public void paintComponent(Graphics g) {
		if (realSize) {
			if (oImage != null && rImage == null) {
				g.drawImage(oImage, 0, 0, this);
			} else if (rImage != null) {
				g.drawImage(rImage, 0, 0, this);
			} else {
				;
			}
		} else {
			scalable = (Graphics2D) g;
			scaling = new AffineTransform();
			scaling.scale(scale, scale);
			scalable.setTransform(scaling);
			if (oImage != null && rImage == null) {
				scalable.drawImage(oImage, 0, 0, this);
			} else if (rImage != null) {
				scalable.drawImage(rImage, 0, 0, this);
			} else {
				;
			}
		}
	}

	public void setImage(String image) {
		oImage = null;
		oImage = this.getToolkit().getImage(image);
		MediaTracker mt = new MediaTracker(this);
		mt.addImage(oImage, 0);
		try {
			mt.waitForAll();
		} catch (InterruptedException e) {
			;
		}
		iWidth = oImage.getWidth(this);
		iHeight = oImage.getHeight(this);
		setPreferredSize(new Dimension(oImage.getWidth(this), oImage
				.getHeight(this)));
	}

	public void setImage(String image, int filter) {
		imageName = image;
		oImage = null;
		oImage = this.getToolkit().getImage(image);
		MediaTracker mt = new MediaTracker(this);
		mt.addImage(oImage, 0);
		try {
			mt.waitForAll();
		} catch (InterruptedException e) {
			;
		}
		iWidth = oImage.getWidth(this);
		iHeight = oImage.getHeight(this);
		setPreferredSize(new Dimension(oImage.getWidth(this), oImage
				.getHeight(this)));
		if (filter == 1) {
			pixels_original = new int[iWidth * iHeight];
			// Pixeldaten einlesen
			PixelGrabber pg = new PixelGrabber(oImage, 0, 0, iWidth, iHeight,
					pixels_original, 0, iWidth);
			try {
				pg.grabPixels();
				convertToGrayscale(pixels_original);
				pixels_before_function = new int[pixels_original.length];
				pixels_after_function = new int[pixels_original.length];
				pixels_result = new int[pixels_original.length];
				// first of all copy the original informations into the working
				// arrays
				copyArrayValues(pixels_original, pixels_before_function);
				copyArrayValues(pixels_original, pixels_after_function);
				rImage = null;
			} catch (InterruptedException e) {
				System.err.println("interrupted waiting for pixels!");
			}
			if (myHistogram == null) {
				myHistogram = new MyHistogram(pixels_before_function);
			} else {
				myHistogram.setHisto(pixels_before_function);
			}
			myHistogram.repaint();
		}
	}

	public void setHistoLocation(int x, int y) {
		if (myHistogram != null) {
			myHistogram.setHistoLocation(x, y);
		}
	}

	public void setHistoSize(int width, int height) {
		if (myHistogram != null) {
			myHistogram.setHistoSize(width, height);
		}
	}
	
	public void setVisibleHistogram(boolean showhisto){
		if(myHistogram!=null){
			myHistogram.setVisible(showhisto);
		}
	}

	public void setJCheckBox(JCheckBox jcb) {
		this.jcb = jcb;
	}

	public double square(double x) {
		return x * x;
	}

	public int[] genFilterMatrix1D(double radius, double sigma) {
		int size = 2 * (int) Math.ceil(radius) + 1;
		int r = (int) Math.ceil(radius);
		float[] erg = new float[size];
		float sum = 0.0f;
		for (int i = 0; i < size; i++) {
			int act = i - r;
			erg[i] = (float) Math.exp(-1.0f
					* ((float) square(act) / (2.0f * (float) square(sigma))));
			System.out.println("Matrix value: " + erg[i]);
			sum = sum + erg[i];
		}
		int[] erg_int = new int[size];
		for (int i = 0; i < size; i++) {
			erg[i] = erg[i] * sum;
			erg_int[i] = Math.round(erg[i]);
		}
		erg = null;
		return erg_int;
	}

	public void runFilter_seperate(double radius, double sigma) {
		// first of all copy the original informations into the working arrays
		copyArrayValues(pixels_original, pixels_before_function);
		copyArrayValues(pixels_original, pixels_after_function);
		int[] filter_matrix = genFilterMatrix1D(radius, sigma);
		// jetzt den Skalierungsfaktor und die
		// Groesse der Matrix
		int scale = 0;
		int matrix_length = filter_matrix.length;
		for (int i = 0; i < matrix_length; i++) {
			scale += filter_matrix[i];
		}
		// jetzt das Koordinatensystem der Matrix
		int u = (matrix_length - 1) / 2;
		int v = (matrix_length - 1) / 2;
		// jetzt die Matrixmultiplikation durchfuehren
		// erstmal horizontal
		int act_pos = 0;
		int act_pixel_value = 0;
		int act_matrix_value = 0;
		int res_multi = 0;
		int sum = 0;

		int x2 = 0;
		int y2 = 0;

		for (int x = 0; x < iWidth - 1; x++) {
			if (x <= u) {
				x2 = u;
			} else if (x >= (iWidth - u - 1)) {
				x2 = iWidth - u - 1;
			} else {
				x2 = x;
			}
			for (int y = 0; y < iHeight - 1; y++) {
				if (y <= v) {
					y2 = v;
				} else if (y >= (iHeight - v - 1)) {
					y2 = iHeight - v - 1;
				} else {
					y2 = y;
				}
				sum = 0;
				for (int u_act = -u; u_act <= u; u_act++) {
					act_pos = y * iWidth + (x2 + u_act);
					act_pixel_value = pixels_after_function[act_pos];
					act_matrix_value = filter_matrix[u_act + u];
					res_multi = act_pixel_value * act_matrix_value;
					sum = sum + res_multi;
				}
				pixels_after_function[y * iWidth + x] = Math.round(sum / scale);
			}
		}
		// in vertikaler Richtung genauso... nur ein wenig anders :-)
		for (int x = 0; x < iWidth - 1; x++) {
			if (x <= u) {
				x2 = u;
			} else if (x >= (iWidth - u - 1)) {
				x2 = iWidth - u - 1;
			} else {
				x2 = x;
			}
			for (int y = 0; y < iHeight - 1; y++) {
				if (y <= v) {
					y2 = v;
				} else if (y >= (iHeight - v - 1)) {
					y2 = iHeight - v - 1;
				} else {
					y2 = y;
				}
				sum = 0;
				for (int v_act = -v; v_act <= v; v_act++) {
					act_pos = (y2 + v_act) * iWidth + x;
					act_pixel_value = pixels_after_function[act_pos];
					act_matrix_value = filter_matrix[v_act + v];
					res_multi = act_pixel_value * act_matrix_value;
					sum = sum + res_multi;
				}
				pixels_after_function[y * iWidth + x] = Math.round(sum / scale);
			}
		}

		// jetzt die Ebene invertieren
		for (int i = 0; i < pixels_after_function.length; i++) {
			pixels_after_function[i] = 255 - pixels_after_function[i];
		}
		// und beide Bilder zusammenfuehren
		for (int i = 0; i < pixels_after_function.length; i++) {
			pixels_after_function[i] = Math
					.round((pixels_before_function[i] + pixels_after_function[i]) / 2);
		}
		copyArrayValues(pixels_after_function, pixels_before_function);
		// copy all values into the array that is used to generate the result
		// image
		copyArrayValues(pixels_after_function, pixels_result);

		if (myHistogram == null) {
			myHistogram = new MyHistogram(pixels_before_function);
		}
		myHistogram.repaint();
		// jetzt ein neues Bild erzeugen
		generateResultImage();
	}

	public void check(int valuemin, int valuemax) {
		for (int i = 0; i < pixels_before_function.length; i++) {
			if ((pixels_before_function[i] >= valuemin)
					&& (pixels_before_function[i] <= valuemax)) {
				pixels_after_function[i] = 255;
			} else {
				pixels_after_function[i] = 0;
			}
		}
		copyArrayValues(pixels_after_function, pixels_result);
		if (myHistogram != null) {
			myHistogram.setMinValue(valuemin);
			myHistogram.setMaxValue(valuemax);
			myHistogram.repaint();
		}
		generateResultImage();
	}

	public void saveImage(String s) {
		pixels_before_function = null;
		pixels_after_function = null;
		System.gc();
		try {
			BufferedImage weg = (BufferedImage) this.createImage(iWidth,
					iHeight);
			Graphics aa = weg.getGraphics();
			if (rImage != null) {
				aa.drawImage(rImage, 0, 0, this);
			}
			ImageIO.write(weg, "png", new File(s));
			System.out.println("image saved");
			weg = null;
		} catch (IOException ioe) {
			System.out.println("can't generate image");
		}
		System.gc();
		pixels_before_function = new int[pixels_original.length];
		pixels_after_function = new int[pixels_original.length];
	}

	public void erode() {
		copyArrayValues(pixels_after_function, pixels_before_function);
		int u = 1;
		int v = 1;
		int act_pos = 0;
		// the number of black pixels
		int sum = 0;
		for (int x = 0 + u; x < iWidth - u; x++) {
			for (int y = 0 + v; y < iHeight - v; y++) {
				sum = 10;
				if (pixels_before_function[(y * iWidth + x)] == 0) {
					sum = 0;
					for (int u_act = -u; u_act <= u; u_act++) {
						for (int v_act = -v; v_act <= v; v_act++) {
							act_pos = (y + v_act) * iWidth + (x + u_act);
							if (pixels_before_function[act_pos] == 0) {
								sum = sum + 1;
							}
						}
					}
				}
				if (sum <= 8) {
					pixels_after_function[(y * iWidth + x)] = 0;
				} else {
					pixels_after_function[(y * iWidth + x)] = 255;
				}
			}
		}
		copyArrayValues(pixels_after_function, pixels_result);
		generateResultImage();
	}

	/**
	 * Thinning algorithm published by Zhang and Suen Communications of the ACM
	 * March 1984 Vol. 27, Number 3
	 */
	public void skeletonizeZHANG2() {
		copyArrayValues(pixels_after_function, pixels_before_function);
		// it is important to have a temporary array that can be edited during
		// processing
		int[] pixels_temp = new int[pixels_after_function.length];
		copyArrayValues(pixels_after_function, pixels_temp);
		MyRemovePixel mrp = new MyRemovePixel();
		// first of all generate a binary encoded image
		convertToBinary(pixels_after_function);
		convertToBinary(pixels_temp);
		int p1, p2, p3, p4, p5, p6, p7, p8, p9;
		int offset = 1;
		int delete = 1;
		while (delete > 0) {
			delete = 0;
			// first iteration
			for (int y = offset; y < (iHeight - offset); y++) {
				for (int x = offset; x < (iWidth - offset); x++) {
					p1 = pixels_after_function[y * iWidth + x];
					if (p1 == 0) {
						pixels_temp[y * iWidth + x] = 0;
						continue;
					} else {
						p9 = pixels_after_function[(y - 1) * iWidth + (x - 1)];
						p2 = pixels_after_function[(y - 1) * iWidth + x];
						p3 = pixels_after_function[(y - 1) * iWidth + (x + 1)];

						p8 = pixels_after_function[y * iWidth + (x - 1)];
						p1 = pixels_after_function[y * iWidth + x];
						p4 = pixels_after_function[y * iWidth + (x + 1)];

						p7 = pixels_after_function[(y + 1) * iWidth + (x - 1)];
						p6 = pixels_after_function[(y + 1) * iWidth + x];
						p5 = pixels_after_function[(y + 1) * iWidth + (x + 1)];

						int[] pixels = { p2, p3, p4, p5, p6, p7, p8, p9 };
						int nBlackPixels = mrp.countBlackPixels(pixels);
						int n01Sequences = mrp.count01Sequence(pixels);
						int r1a = p2 * p4 * p6;
						int r1b = p4 * p6 * p8;
						if ((nBlackPixels >= 2 && nBlackPixels <= 6)
								&& n01Sequences == 1 && r1a == 0 && r1b == 0) {
							pixels_temp[y * iWidth + x] = 1;
							delete++;
						} else {
							pixels_temp[y * iWidth + x] = 0;
						}
					}
				}
			}
			// substract images
			for (int i = 0; i < pixels_after_function.length; i++) {
				pixels_after_function[i] -= pixels_temp[i];
			}
			if (delete == 0) {
				break;
			}
			// second iteration
			for (int y = offset; y < (iHeight - offset); y++) {
				for (int x = offset; x < (iWidth - offset); x++) {
					p1 = pixels_after_function[y * iWidth + x];
					if (p1 == 0) {
						pixels_temp[y * iWidth + x] = 0;
						continue;
					} else {
						p9 = pixels_after_function[(y - 1) * iWidth + (x - 1)];
						p2 = pixels_after_function[(y - 1) * iWidth + x];
						p3 = pixels_after_function[(y - 1) * iWidth + (x + 1)];

						p8 = pixels_after_function[y * iWidth + (x - 1)];
						p1 = pixels_after_function[y * iWidth + x];
						p4 = pixels_after_function[y * iWidth + (x + 1)];

						p6 = pixels_after_function[(y + 1) * iWidth + x];
						p7 = pixels_after_function[(y + 1) * iWidth + (x - 1)];
						p5 = pixels_after_function[(y + 1) * iWidth + (x + 1)];

						int[] pixels = { p2, p3, p4, p5, p6, p7, p8, p9 };
						int nBlackPixels = mrp.countBlackPixels(pixels);
						int n01Sequences = mrp.count01Sequence(pixels);
						int r1a = p2 * p4 * p8;
						int r1b = p2 * p6 * p8;
						if ((nBlackPixels >= 2 && nBlackPixels <= 6)
								&& n01Sequences == 1 && r1a == 0 && r1b == 0) {
							pixels_temp[y * iWidth + x] = 1;
							delete++;
						} else {
							pixels_temp[y * iWidth + x] = 0;
						}
					}
				}
			}
			// substract images
			for (int i = 0; i < pixels_after_function.length; i++) {
				pixels_after_function[i] -= pixels_temp[i];
			}
		}
		pixels_temp = null;
		// convert to 8 bit image
		convertTo8Bit(pixels_after_function);
		copyArrayValues(pixels_after_function, pixels_result);
		generateResultImage();
	}

	public void dilate() {
		copyArrayValues(pixels_after_function, pixels_before_function);
		int u = 1;
		int v = 1;
		int act_pos = 0;
		// the number of black pixels
		int sum = 0;
		for (int x = 0 + u; x < iWidth - u; x++) {
			for (int y = 0 + v; y < iHeight - v; y++) {
				sum = 0;
				for (int u_act = -u; u_act <= u; u_act++) {
					for (int v_act = -v; v_act <= v; v_act++) {
						act_pos = (y + v_act) * iWidth + (x + u_act);
						if (pixels_before_function[act_pos] == 0) {
							sum++;
						}
					}
				}
				if (sum > 3) {
					pixels_after_function[(y * iWidth + x)] = 0;
				}
			}
		}
		copyArrayValues(pixels_after_function, pixels_result);
		generateResultImage();
	}

	/**
	 * logb(x) = loga(x) / loga(b)
	 * 
	 * @param x
	 * @return
	 */
	public static double logBase2(double x) {
		return Math.log(x) / Math.log(2.0);
	}

	private void convertToGrayscale(int[] c) {
		// if something is wrong, have a closer look at the operations performed
		// on the arrays
		float blue = 0.0f;
		float red = 0.0f;
		float green = 0.0f;
		float gray = 0.0f;
		for (int i = 0; i < c.length; i++) {
			gray = 0.0f;
			blue = (c[i]) & 0xff;
			green = (c[i] >> 8) & 0xff;
			red = (c[i] >> 16) & 0xff;
			gray = red * 0.2125f + green * 0.7154f + blue * 0.072f;
			c[i] = Math.round(gray);
		}
	}

	private void generateResultImage() {
		System.gc();
		// convert into RGB
		for (int i = 0; i < pixels_result.length; i++) {
			int gray2 = pixels_result[i];
			pixels_result[i] = 0xff000000 | (gray2 << 16) | (gray2 << 8)
					| gray2;
		}
		MemoryImageSource mis = new MemoryImageSource(iWidth, iHeight,
				ColorModel.getRGBdefault(), pixels_result, 0, iWidth);
		rImage = createImage(mis);
		repaint();
	}

	/**
	 * copy the values from the source into the target array
	 * 
	 * @param source
	 * @param target
	 */
	private void copyArrayValues(int[] source, int[] target) {
		for (int i = 0; i < source.length; i++) {
			target[i] = source[i];
		}
	}

	public void undo() {
		copyArrayValues(pixels_before_function, pixels_after_function);
		copyArrayValues(pixels_before_function, pixels_result);
		generateResultImage();
	}

	private void convertToBinary(int[] c) {
		for (int i = 0; i < c.length; i++) {
			if (c[i] == 255) {
				c[i] = 0;
			} else {
				c[i] = 1;
			}
		}
	}

	private void convertTo8Bit(int[] c) {
		for (int i = 0; i < c.length; i++) {
			if (c[i] == 0) {
				c[i] = 255;
			} else {
				c[i] = 0;
			}
		}
	}

	public void disposeMyHistogram() {
		if (myHistogram != null) {
			myHistogram.dispose();
		}
	}

	public void countPixels() {
		// System.out.println("countPixels aufgerufen...");
		if ((n_Laenge_X == 0) || (n_Laenge_Y == 0) || (n_Pixel_X == 0)
				|| (n_Pixel_Y == 0)) {
			System.out.println("not calibrated");
		}
		int sumWhite = 0;
		int sumBlack = 0;
		int sum = pixels_after_function.length;
		for (int i = 0; i < pixels_after_function.length; i++) {
			if (pixels_after_function[i] == 0) {
				sumBlack++;
			} else {
				sumWhite++;
			}
		}
		System.out.println(imageName + " black: " + sumBlack + " white: "
				+ sumWhite + " ges: " + sum);
		File f = new File(imageName);
		String filename = f.getParent() + File.separator
				+ f.getName().substring(0, f.getName().length() - 4)
				+ "_pixels.txt";

		// area calculated as defined
		double gesamt_cal = (n_Laenge_X / n_Pixel_X) * (n_Laenge_Y / n_Pixel_Y)
				* sum;
		double gesamt_black = (n_Laenge_X / n_Pixel_X)
				* (n_Laenge_Y / n_Pixel_Y) * sumBlack;
		double gesamt_white = (n_Laenge_X / n_Pixel_X)
				* (n_Laenge_Y / n_Pixel_Y) * sumWhite;

		try {
			FileWriter file_out = new FileWriter(filename);
			file_out
					.write("black\twhite\tsum\tarea_black\tarea_white\tsum_area\timage\n");
			file_out.write(sumBlack + "\t" + sumWhite + "\t"
					+ (sumBlack + sumWhite) + "\t" + gesamt_black + "\t"
					+ gesamt_white + "\t" + gesamt_cal + "\t" + f.getName()
					// .substring(0, f.getName().length() - 4)
					+ "\n");

			// file_out
			// .write(gesamt_black
			// + "\t"
			// + gesamt_white
			// + "\t"
			// + gesamt_cal
			// + "\t"
			// + f.getName()
			// .substring(0, f.getName().length() - 4)
			// + "\n");
			file_out.close();
			System.out.println("number of pixels saved on harddisc: "
					+ filename);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void setCalibration(double x_pixel, double x_length, double y_pixel,
			double y_length) {
		this.n_Pixel_X = x_pixel;
		this.n_Laenge_X = x_length;
		this.n_Pixel_Y = y_pixel;
		this.n_Laenge_Y = y_length;
		System.out.println("calibration: " + n_Pixel_X + " " + n_Laenge_X + " "
				+ n_Pixel_Y + " " + n_Laenge_Y);
	}
}

class MyRemovePixel {
	// the selected pixel is at the center of the 5x5 matrix
	// using the following arrays the centered pixel is at position 0
	// value = 1 indicates that this pixel has to be set
	// value = 0 no pixel
	// value = -1 don't care about the value of the pixel
	public MyRemovePixel() {
		;
	}

	// just remember: 0 = black, 255 = white
	public int count255_0Sequence(int[] seq) {
		int a = 0;
		for (int i = 1; i < seq.length; i++) {
			if ((seq[i - 1] == 255) && (seq[i] == 0)) {
				a++;
			}
		}
		if (a == 0 && seq[0] == 0) {
			a = 1;
		}
		return a;
	}

	public int countBlackPixels(int[] seq) {
		int a = 0;
		for (int i = 0; i < seq.length; i++) {
			if (seq[i] == 1)
				a++;
		}
		return a;
	}

	/**
	 * 
	 * @param seq
	 *            8 neighbor pixels
	 * @return
	 */
	public int count01Sequence(int[] seq) {
		int a = 0;
		for (int i = 1; i < seq.length; i++) {
			if ((seq[i - 1] == 0) && (seq[i] == 1)) {
				a++;
			}
		}
		if (seq[0] == 1 && seq[seq.length - 1] == 0)
			a++;
		return a;
	}

	public static int compareTwoArrays(int[] arrayOriginal, int[] arrayToCompare) {
		int diff = -1;
		if (arrayOriginal.length == arrayToCompare.length) {
			diff = 0;
			for (int i = 0; i < arrayOriginal.length; i++) {
				if (arrayOriginal[i] != arrayToCompare[i]) {
					diff++;
				}
				;
			}
		}
		return diff;
	}
}

class MyHistogram extends JFrame {

	private static final long serialVersionUID = 1L;

	private int[] histo = null;

	private JPanel jContentPane = null;

	private MyHistoPanel myHistoPanel = null;

	private MyColorGradient jPanel = null;

	/**
	 * This is the default constructor
	 */
	public MyHistogram(int[] histo) {
		super();
		this.histo = histo;
		initialize();
		pack();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
//		this.setSize(300, 200);
		this.setContentPane(getJContentPane());
		this.setTitle("histogram");
		this.setVisible(true);
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
			jContentPane.add(getMyHistoPanel(), BorderLayout.CENTER);
			jContentPane.add(getJPanel(), BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	private MyHistoPanel getMyHistoPanel() {
		if (myHistoPanel == null) {
			myHistoPanel = new MyHistoPanel(histo);
			myHistoPanel.setBorder(BorderFactory.createLineBorder(Color.white,
					1));
		}
		return myHistoPanel;
	}

	public void setMinValue(int minValue) {
		myHistoPanel.setMinValue(minValue);
	}

	public void setMaxValue(int maxValue) {
		myHistoPanel.setMaxValue(maxValue);
	}

	public void setHisto(int[] histo) {
		this.histo = histo;
		myHistoPanel.setHisto(histo);
	}

	/**
	 * This method initializes jPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridy = 0;
			gridBagConstraints1.anchor = GridBagConstraints.WEST;
			gridBagConstraints1.gridx = 0;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 1;
			gridBagConstraints.anchor = GridBagConstraints.CENTER;
			gridBagConstraints.gridy = 0;
			jPanel = new MyColorGradient();
			jPanel.setLayout(new GridBagLayout());
		}
		return jPanel;
	}

	public void setHistoLocation(int x, int y) {
		this.setLocation(x, y);
	}

	public void setHistoSize(int width, int height) {
		this.setSize(width, height);
	}

}

class MyColorGradient extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private int width = 0;
	private int height = 0;
	private Graphics2D g2d = null;

	@Override
	public Dimension getPreferredSize() {
		return new Dimension(40, 10);
	}

	@Override
	protected void paintComponent(Graphics g) {
		width = this.getWidth();
		height = this.getHeight();
		g2d = (Graphics2D) g;
		GradientPaint gradpaint = new GradientPaint(0, 0, Color.black, width,
				0, Color.white);
		g2d.setPaint(gradpaint);
		g2d.fillRect(0, 0, width, height);
	}
}

class MyHistoPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private int[] histoData = null;

	private int minValue = 0;

	private int maxValue = 0;

	private int maxCount = 0;

	// private int panelWidth = 80;
	private int panelWidth = 255;

	private int panelHeight = 80;

	private int[] histo = null;

	/**
	 * @param maxValue
	 *            the maxValue to set
	 */
	public void setMaxValue(int maxValue) {
		this.maxValue = maxValue;
	}

	/**
	 * @param minValue
	 *            the minValue to set
	 */
	public void setMinValue(int minValue) {
		this.minValue = minValue;
	}

	public MyHistoPanel(int[] histo) {
		this.histo = histo;
		this.histoData = new int[256];
		for (int i = 0; i < this.histoData.length; i++) {
			this.histoData[i] = 0;
		}
		maxCount = 0;
		generateHisto();
	}

	public void setHisto(int[] histo) {
		this.histo = histo;
		for (int i = 0; i < this.histoData.length; i++) {
			this.histoData[i] = 0;
		}
		maxCount = 0;
		generateHisto();
	}

	@Override
	protected void paintComponent(Graphics g) {
		this.panelWidth = this.getWidth();
		this.panelHeight = this.getHeight();

		generateHisto();
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, panelWidth, panelHeight);
		g.setColor(Color.BLACK);
		g.fillRect(0, 0, cScale(minValue), panelHeight);
		g.fillRect(cScale(maxValue), 0, panelWidth, panelHeight);
		g.setColor(Color.GRAY);
		for (int i = 0; i < histoData.length; i++) {
			g.drawLine(cScale(i), panelHeight, cScale(i),
					(panelHeight - histoData[i]));
		}
		g.setColor(Color.RED);
		g.drawLine(cScale(minValue), 0, cScale(minValue), panelHeight);
		g.drawLine(cScale(maxValue), 0, cScale(maxValue), panelHeight);
		g.setColor(Color.BLACK);
	}

	private int cScale(int value) {
		return Math.round(panelWidth / 256f * value);
	}

	private void generateHisto() {
		int actualValue = 0;
		for (int i = 0; i < histo.length; i++) {
			actualValue = histo[i];
			histoData[actualValue] = histoData[actualValue] + 1;
			if (maxCount < histoData[actualValue]) {
				maxCount = histoData[actualValue];
			}
		}
		// scale histoData
		for (int i = 0; i < histoData.length; i++) {
			histoData[i] = Math.round((float) panelHeight / (float) maxCount
					* histoData[i]);
		}
	}

	@Override
	public Dimension getPreferredSize() {
		return new Dimension(this.panelWidth, this.panelHeight);
	}
}