import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.media.*;
import javax.media.control.FormatControl;
import javax.media.control.FrameGrabbingControl;
import javax.media.format.RGBFormat;
import javax.media.format.VideoFormat;
import javax.media.format.YUVFormat;
import javax.media.protocol.CaptureDevice;
import javax.media.protocol.DataSource;
import javax.media.util.BufferToImage;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JButton;

import javax.swing.JTextField;
import javax.swing.JLabel;

/**
 * @author Markus Boettcher
 * 
 */
public class MyGrabber extends JFrame {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * gewuenschte Aufloesung und Bandbreite
	 */
	Dimension aufl = new Dimension(640, 480);

	// Dimension aufl = new Dimension(160, 120);

	// aufl.width = 640;
	// aufl.height = 480;
	// int bandwidth = 460800;
	int bandwidth = 61440;
	// int bandwidth = 28800;

	// int bandwidth = 614400;
	/**
	 * Die Zahl der zu speichernden Frames
	 */
	int nFrames = 5;

	/**
	 * Die Zeit zwischen den einzelnen Aufnahmen in mS
	 */
	int timeBetweenFrames = 1000;

	int i = 0;

	/**
	 * hier der framegrabber
	 */
	FrameGrabbingControl grabFrame = null;

	/**
	 * der Puffer fuer den Framegrabber
	 */
	Buffer buffer = null;

	/**
	 * das gegrabbte Bild
	 */
	Image image = null;

	/**
	 * ImageBuffer fuer image
	 */
	BufferedImage buffImage = null;

	/**
	 * image in buffImage einzeichnen
	 */
	Graphics2D g2d = null;

	DataSource dataSource = null; // @jve:decl-index=0:

	CaptureDeviceInfo grabber = null; // @jve:decl-index=0:

	Player player = null; // @jve:decl-index=0:

	Component visualComponent = null;

	Component control = null;

	private JPanel jContentPane = null;

	private JPanel buttonPanel = null;

	private JButton capture = null;

	private JTextField filePrefix = null;

	private JLabel jLabel = null;

	private File workingDirectory = null;

	/**
	 * This method initializes buttonPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getButtonPanel() {
		if (buttonPanel == null) {
			jLabel = new JLabel();
			jLabel.setText("file prefix:");
			buttonPanel = new JPanel();
			buttonPanel.add(jLabel, null);
			buttonPanel.add(getFilePrefix(), null);
			buttonPanel.add(getCapture(), null);
		}
		return buttonPanel;
	}

	/**
	 * This method initializes capture
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getCapture() {
		if (capture == null) {
			capture = new JButton();
			capture.setText("Save");
			capture.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					// if more than one image should be acquired
					// for (int i = 0; i <= nFrames; i++) {
					// try {
					// grabFrame();
					// Thread.sleep(timeBetweenFrames);
					// } catch (Exception e2) {
					// ;
					// }
					// }
					grabFrame();
				}
			});
		}
		return capture;
	}

	/**
	 * This method initializes filePrefix
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getFilePrefix() {
		if (filePrefix == null) {
			filePrefix = new JTextField();
			filePrefix.setPreferredSize(new java.awt.Dimension(100, 20));
			filePrefix.setText("a");
		}
		return filePrefix;
	}

	/**
	 * @param args
	 */
	// public static void main(String[] args) {
	// // TODO Auto-generated method stub
	// MyGrabber myGrabber = new MyGrabber();
	// }
	/**
	 * This is the default constructor
	 */
	public MyGrabber() {
		super();
		initialize();
		initializeJMF();
		pack();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(300, 200);
		this.setContentPane(getJContentPane());
		this.setTitle("Image acquisition");
		this.setVisible(true);
		this.addWindowListener(new java.awt.event.WindowAdapter() {
			@Override
			public void windowClosing(java.awt.event.WindowEvent e) {
				System.out.println("windowClosing()");
				if (player != null) {
					player.stop();
					player.close();
					player.deallocate();
				}
				// System.exit(0);
				dispose();
			}
		});
		JFileChooser selectDirectory = new JFileChooser();
		selectDirectory.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		selectDirectory.setDialogType(JFileChooser.CUSTOM_DIALOG);
		selectDirectory.setDialogTitle("select working directory");
		selectDirectory.setApproveButtonText("select");
		selectDirectory.setAcceptAllFileFilterUsed(false);
		selectDirectory.showDialog(new JFrame(), null);
		workingDirectory = selectDirectory.getSelectedFile();
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getButtonPanel(), java.awt.BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	private void initializeJMF() {
		Manager.setHint(Manager.LIGHTWEIGHT_RENDERER, new Boolean(true));
		Vector deviceList = CaptureDeviceManager.getDeviceList(new VideoFormat(
				VideoFormat.YUV));
		if (deviceList.size() > 0) {
			/**
			 * Erstmal die Hardware suchen, die das YUV-Format unterstuetzt
			 */
			for (int i = 0; i < deviceList.size(); i++) {
				System.out.println("Ok... Grabber gefunden");
				System.out.println("Was wurde gefunden: "
						+ ((CaptureDeviceInfo) deviceList.elementAt(i))
								.getName());
				if (deviceList.size() == 1) {
					System.out
							.println("Hurra... es wurde auch nur ein Grabber gefunden...");
				} else {
					System.out
							.println("An dieser Stelle nochmal nachdenken, es wurden mehrere Grabber gefunden");
				}
			}
			grabber = CaptureDeviceManager
					.getDevice(((CaptureDeviceInfo) deviceList.firstElement())
							.getName());
			// grabber = CaptureDeviceManager
			// .getDevice(((CaptureDeviceInfo) deviceList.elementAt(1))
			// .getName());

			/**
			 * die Formate abfragen
			 */
			Format[] formats = grabber.getFormats();

			// for(int i=0; i<formats.length;i++){
			// System.out.println("Format...: "+formats[i].toString());
			// }

			YUVFormat yuvFormat = null;

			for (int i = 0; i < formats.length; i++) {
				if (formats[i] instanceof YUVFormat) {
					yuvFormat = (YUVFormat) formats[i];
					Dimension d = yuvFormat.getSize();
					if (d.equals(aufl)
							&& bandwidth == yuvFormat.getMaxDataLength()) {
						System.out.println("Aufloesung: " + d.width + " x "
								+ d.height);
						System.out.println("MaxDataLength: "
								+ yuvFormat.getMaxDataLength());
						break;
					}
					yuvFormat = null;
				}
			}
			try {
				dataSource = Manager.createDataSource(grabber.getLocator());

				FormatControl[] formatControls = ((CaptureDevice) dataSource)
						.getFormatControls();
				Format format = null;

				for (int i = 0; i < formatControls.length; i++) {
					if (formatControls[i] == null) {
						continue;
					}
					if ((format = formatControls[i].setFormat(yuvFormat)) != null) {

						System.out.println("..." + format.toString());
						break;
					}
				}

				player = Manager.createRealizedPlayer(dataSource);
				// player = Manager.createRealizedPlayer(grabber.getLocator());
				player.start();
				if ((visualComponent = player.getVisualComponent()) != null) {
					visualComponent.setSize(aufl);
					this.add(visualComponent, BorderLayout.CENTER);
					// uncomment if the visualComponent should react on mouse
					// events
					// visualComponent
					// .addMouseListener(new java.awt.event.MouseListener() {
					// public void mouseClicked(
					// java.awt.event.MouseEvent e) {
					// grabFrame();
					// }
					//
					// public void mousePressed(
					// java.awt.event.MouseEvent e) {
					// }
					//
					// public void mouseReleased(
					// java.awt.event.MouseEvent e) {
					// }
					//
					// public void mouseEntered(
					// java.awt.event.MouseEvent e) {
					// }
					//
					// public void mouseExited(
					// java.awt.event.MouseEvent e) {
					// }
					// });
				}
				// if ((control = player.getControlPanelComponent())!=null){
				// this.add(control, BorderLayout.SOUTH);
				// }
				// Control[] pc = player.getControls();
				// for (int i = 0; i < pc.length; i++) {
				// System.out.println("control " + pc[i].toString());
				// }
			} catch (Exception e) {
				System.out.println("Hier luft was schief... " + e);
			}
			try {
				Thread.sleep(2500);
			} catch (Exception e) {
				;
			}

		}
	}

	private void grabFrame() {
		i++;
		grabFrame = (FrameGrabbingControl) player
				.getControl("javax.media.control.FrameGrabbingControl");
		buffer = grabFrame.grabFrame();
		BufferToImage vf = new BufferToImage((RGBFormat) buffer.getFormat());
		image = vf.createImage(buffer);

		buffImage = new BufferedImage(image.getWidth(null), image
				.getHeight(null), BufferedImage.TYPE_INT_RGB);

		g2d = buffImage.createGraphics();
		g2d.drawImage(image, 0, 0, this);

		/**
		 * das erstellte Bild speichern
		 */
		try {
			ImageIO.write(buffImage, "png", new File(workingDirectory
					+ File.separator + filePrefix.getText() + i + ".png"));
		} catch (Exception e) {
			System.out.println("..." + e.toString());
		}

	}
}
